import { Directive, Input, HostListener, ElementRef, Pipe, forwardRef, InjectionToken, Inject, EventEmitter, Output, NgModule } from '@angular/core';
import { NG_VALIDATORS } from '@angular/forms';
import { Observable } from 'rxjs/Observable';
import 'rxjs/add/operator/map';
import 'rxjs/add/observable/fromPromise';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const CEP_MASK = '_____-___';
const CNPJ_MASK = '__.___.___/____-__';
const CPF_MASK = '___.___.___-__';
class StringFormatter {
    /**
     * @param {?} input
     * @param {?} mask
     * @return {?}
     */
    static maskedFormatter(input, mask) {
        // O caracter que representa máscara deve ser o caracter mais comum na string
        // os demais são de formatação, então retorno o caracter mais comum
        const /** @type {?} */ maskChr = StringFormatter.getMostCommonChar(mask);
        // Tranfomando a máscara em um array, mais fácil
        // para formatar o texto
        const /** @type {?} */ maskArr = mask.split('');
        // Limpando a formatação antes de aplicar a máscara
        // e tranforamando em um array, pois simplifica o código
        const /** @type {?} */ inputArr = StringFormatter.clearFormat(input, mask).split('');
        let /** @type {?} */ currMaskChar = '';
        let /** @type {?} */ newValue = '';
        while (inputArr.length > 0 && maskArr.length > 0) {
            currMaskChar = maskArr.shift();
            // Se o caracter da máscara for #, adiciona
            // um caracter do inputArr a string formatada
            // e remove o caracter do maskArr
            if (currMaskChar === maskChr) {
                newValue += inputArr.shift();
            }
            else {
                // Se o caracter da máscara não for #, adiciona esse
                // ao string formatada
                newValue += currMaskChar;
            }
        }
        return newValue + maskArr.join('');
    }
    /**
     * @param {?} str
     * @param {?} mask
     * @return {?}
     */
    static clearFormat(str, mask) {
        // O caracter que representa máscara deve ser o caracter mais comum na string
        // os demais são de formatação, então retorno o caracter mais comum
        const /** @type {?} */ maskChr = StringFormatter.getMostCommonChar(mask);
        // Obtém os caracteres de formatação, removento os de máscara,
        // que vai ser usado para limpar a formatação
        const /** @type {?} */ maskStr = mask.replace(maskChr, '').split('');
        // Fazendo uma cópia do input
        let /** @type {?} */ retVal = str;
        for (const /** @type {?} */ c of maskStr) {
            retVal = retVal.replace(c, '');
        }
        return retVal;
    }
    /**
     * @param {?} str
     * @return {?}
     */
    static getMostCommonChar(str) {
        const /** @type {?} */ strCountArr = new Map();
        let /** @type {?} */ maxKey = '';
        for (const /** @type {?} */ key of str) {
            if (!strCountArr.has(key)) {
                strCountArr.set(key, 1);
            }
            else {
                strCountArr.set(key, strCountArr.get(key) + 1);
            }
            if (maxKey === '' || strCountArr.get(maxKey) < strCountArr.get(key)) {
                maxKey = key;
            }
        }
        return maxKey;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class MaskPatternDirective {
    /**
     * @param {?} el
     */
    constructor(el) {
        this.el = el;
        // tslint:disable-next-line:no-input-rename
        this.showPlaceholder = true;
        this.patternStr = [];
        this.patternArr = [];
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this.showPlaceholder && !this.el.nativeElement.placeholder) {
            this.el.nativeElement.placeholder = this.pattern;
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onInput(event) {
        const /** @type {?} */ input = (/** @type {?} */ (event.target));
        input.value = StringFormatter.maskedFormatter(input.value, this.pattern);
    }
}
MaskPatternDirective.decorators = [
    { type: Directive, args: [{
                // tslint:disable-next-line:directive-selector
                selector: '[ngBrToolsMaskPattern]'
            },] },
];
/** @nocollapse */
MaskPatternDirective.ctorParameters = () => [
    { type: ElementRef, },
];
MaskPatternDirective.propDecorators = {
    "pattern": [{ type: Input, args: ['ngBrToolsMaskPattern',] },],
    "showPlaceholder": [{ type: Input, args: ['ngBrToolsShowPlacholder',] },],
    "onInput": [{ type: HostListener, args: ['input', ['$event'],] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class MaskPatternPipe {
    /**
     * @param {?} value
     * @param {?} mask
     * @return {?}
     */
    transform(value, mask) {
        return StringFormatter.maskedFormatter(value, mask);
    }
}
MaskPatternPipe.decorators = [
    { type: Pipe, args: [{
                name: 'ngBrToolsMaskPatternPipe'
            },] },
];
/** @nocollapse */
MaskPatternPipe.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class CpfPatternDirective extends MaskPatternDirective {
    /**
     * @return {?}
     */
    ngOnInit() {
        this.pattern = CPF_MASK;
        super.ngOnInit();
    }
}
CpfPatternDirective.decorators = [
    { type: Directive, args: [{
                // tslint:disable-next-line:directive-selector
                selector: '[ngBrToolsCpfPattern]'
            },] },
];
/** @nocollapse */
CpfPatternDirective.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class CpfPatternPipe {
    /**
     * @param {?} value
     * @return {?}
     */
    transform(value) {
        return StringFormatter.maskedFormatter(value, CPF_MASK);
    }
}
CpfPatternPipe.decorators = [
    { type: Pipe, args: [{
                name: 'ngBrToolsCpfPatternPipe'
            },] },
];
/** @nocollapse */
CpfPatternPipe.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class BrValidator {
    /**
     * @param {?} cpf
     * @return {?}
     */
    static validaCpf(cpf) {
        let /** @type {?} */ cpfAux = '';
        let /** @type {?} */ digito1 = '';
        let /** @type {?} */ digito2 = '';
        const /** @type {?} */ brValidatorUtils = new BrValidatotUtils(cpf);
        if (!brValidatorUtils.isValid()) {
            return false;
        }
        cpfAux = brValidatorUtils.removeDigitosVerificador();
        digito1 = brValidatorUtils.calculaDigitoVerificador(cpfAux);
        digito2 = brValidatorUtils.calculaDigitoVerificador(cpfAux + digito1);
        return cpf === cpfAux + digito1 + digito2;
    }
    /**
     * @param {?} cnpj
     * @return {?}
     */
    static validaCnpj(cnpj) {
        let /** @type {?} */ cnpjAux = '';
        let /** @type {?} */ digito1 = '';
        let /** @type {?} */ digito2 = '';
        const /** @type {?} */ brValidatorUtils = new BrValidatotUtils(cnpj);
        if (!brValidatorUtils.isValid()) {
            return false;
        }
        cnpjAux = brValidatorUtils.removeDigitosVerificador();
        digito1 = brValidatorUtils.calculaDigitoVerificador(cnpjAux);
        digito2 = brValidatorUtils.calculaDigitoVerificador(cnpjAux + digito1);
        return cnpj === cnpjAux + digito1 + digito2;
    }
}
class BrValidatotUtils {
    /**
     * @param {?} str
     */
    constructor(str) {
        this.str = str;
    }
    /**
     * @return {?}
     */
    isValid() {
        return this.checkCanBeValid(this.str, 'cpf') ||
            this.checkCanBeValid(this.str, 'cnpj');
    }
    /**
     * @param {?} str
     * @return {?}
     */
    calculaDigitoVerificador(str) {
        let /** @type {?} */ tblPesos = [];
        let /** @type {?} */ digito;
        let /** @type {?} */ somatorio = 0;
        // Definindo que tabela de pesos
        // devo usar, cnpj ou cpf
        if (str.length === 9) {
            // CPF
            tblPesos = [10, 9, 8, 7, 6, 5, 4, 3, 2];
        }
        else if (str.length === 10) {
            // CPF + primeiro digito verificador
            tblPesos = [11, 10, 9, 8, 7, 6, 5, 4, 3, 2];
        }
        else if (str.length === 12) {
            // CNPJ
            tblPesos = [5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2];
        }
        else if (str.length === 13) {
            // CNPJ + primeiro dígito verificador
            tblPesos = [6, 5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2];
        }
        // Faz o somatório dos digitos * posição tabela de pesos
        let /** @type {?} */ i = 0;
        for (const /** @type {?} */ p of tblPesos) {
            somatorio += p * Number(str.charAt(i));
            i++;
        }
        // Divide o somatório por 11 e obtém o resto
        const /** @type {?} */ resto = somatorio % 11;
        // Se o resto for menor que 2, então o dígito é zero
        // sentão  o ditigo vai ser igual ao 11 - resto
        // Então se o 11 - resto > 9  o digito = 0 (que dá no mesmo)
        digito = 11 - resto;
        if (digito > 9) {
            digito = 0;
        }
        return digito.toString();
    }
    /**
     * @return {?}
     */
    removeDigitosVerificador() {
        return this.str.substring(0, this.str.length - 2);
    }
    /**
     * @param {?} value
     * @param {?} opt
     * @return {?}
     */
    checkCanBeValid(value, opt) {
        if (value == null || value.length === 0) {
            return false;
        }
        switch (opt) {
            case 'cpf':
                return this.canBeValidCpf(value);
            case 'cnpj':
                return this.canBeValidCnpj(value);
        }
        return false;
    }
    /**
     * @param {?} cpf
     * @return {?}
     */
    canBeValidCpf(cpf) {
        if (cpf.length !== 11) {
            return false;
        }
        if ((cpf === '00000000000') ||
            (cpf === '11111111111') ||
            (cpf === '22222222222') ||
            (cpf === '33333333333') ||
            (cpf === '44444444444') ||
            (cpf === '55555555555') ||
            (cpf === '66666666666') ||
            (cpf === '77777777777') ||
            (cpf === '88888888888') ||
            (cpf === '99999999999')) {
            return false;
        }
        return true;
    }
    /**
     * @param {?} cnpj
     * @return {?}
     */
    canBeValidCnpj(cnpj) {
        if (cnpj.length !== 14) {
            return false;
        }
        if ((cnpj === '00000000000000') ||
            (cnpj === '11111111111111') ||
            (cnpj === '22222222222222') ||
            (cnpj === '33333333333333') ||
            (cnpj === '44444444444444') ||
            (cnpj === '55555555555555') ||
            (cnpj === '66666666666666') ||
            (cnpj === '77777777777777') ||
            (cnpj === '88888888888888') ||
            (cnpj === '99999999999999')) {
            return false;
        }
        return true;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
// tslint:disable-next-line:directive-class-suffix
class CPFValidator {
    /**
     * @param {?} c
     * @return {?}
     */
    validate(c) {
        if (c.value && c.value.length > 0) {
            const /** @type {?} */ regex = /\.|-/gi;
            const /** @type {?} */ value = c.value.replace(regex, '');
            if (BrValidator.validaCpf(value)) {
                return null;
            }
        }
        return {
            CPFValidator: {
                valid: false
            }
        };
    }
}
CPFValidator.decorators = [
    { type: Directive, args: [{
                // tslint:disable-next-line:directive-selector
                selector: '[ngBrToolsCpfValidator][ngModel],[ngBrToolsCpfValidator][formControl],[ngBrToolsCpfValidator][formControlName]',
                providers: [
                    {
                        provide: NG_VALIDATORS,
                        useExisting: forwardRef(() => CPFValidator),
                        multi: true
                    }
                ]
            },] },
];
/** @nocollapse */
CPFValidator.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class CnpjPatternDirective extends MaskPatternDirective {
    /**
     * @return {?}
     */
    ngOnInit() {
        this.pattern = CNPJ_MASK;
        super.ngOnInit();
    }
}
CnpjPatternDirective.decorators = [
    { type: Directive, args: [{
                // tslint:disable-next-line:directive-selector
                selector: '[ngBrToolsCnpjPattern]'
            },] },
];
/** @nocollapse */
CnpjPatternDirective.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class CnpjPatternPipe {
    /**
     * @param {?} value
     * @return {?}
     */
    transform(value) {
        return StringFormatter.maskedFormatter(value, CNPJ_MASK);
    }
}
CnpjPatternPipe.decorators = [
    { type: Pipe, args: [{
                name: 'ngBrToolsCnpjPatternPipe'
            },] },
];
/** @nocollapse */
CnpjPatternPipe.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
// tslint:disable-next-line:directive-class-suffix
class CNPJValidator {
    /**
     * @param {?} c
     * @return {?}
     */
    validate(c) {
        if (c.value && c.value.length > 0) {
            const /** @type {?} */ regex = /\.|-|\//gi;
            const /** @type {?} */ value = c.value.replace(regex, '');
            if (BrValidator.validaCnpj(value)) {
                return null;
            }
        }
        return {
            CNPJValidator: {
                valid: false
            }
        };
    }
}
CNPJValidator.decorators = [
    { type: Directive, args: [{
                // tslint:disable-next-line:directive-selector
                selector: '[ngBrToolsCnpjValidator][ngModel],[ngBrToolsCnpjValidator][formControl],[ngBrToolsCnpjValidator][formControlName]',
                providers: [
                    {
                        provide: NG_VALIDATORS,
                        useExisting: forwardRef(() => CNPJValidator),
                        multi: true
                    }
                ]
            },] },
];
/** @nocollapse */
CNPJValidator.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
let ESTADO_SERVICE = new InjectionToken('estado.service');
let EstadoServiceFactory = (estadoService) => {
    estadoService.init();
    return estadoService;
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class LstEstadoDirective {
    /**
     * @param {?} el
     * @param {?} estadoService
     */
    constructor(el, estadoService) {
        this.el = el;
        // tslint:disable-next-line:no-input-rename
        this.orderBy = 'nome';
        // tslint:disable-next-line:no-output-rename
        this.onEstados = new EventEmitter();
        this._estadoService = estadoService;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        // Carregando os estados
        let /** @type {?} */ resp = this._estadoService.buscaEstados();
        if (resp instanceof Promise) {
            resp = Observable.fromPromise((/** @type {?} */ (resp)));
        }
        (/** @type {?} */ (resp)).map((estados) => {
            return this._sortEstados(estados, this.orderBy);
        }).subscribe((e) => this.onEstados.emit(e));
    }
    /**
     * @param {?} lst
     * @param {?=} por
     * @return {?}
     */
    _sortEstados(lst, por = 'nome') {
        if (lst.length <= 1) {
            return lst;
        }
        if (por === 'nome') {
            return lst.slice(0).sort((e1, e2) => {
                if (e1.nome === e2.nome) {
                    return 0;
                }
                return (e1.nome < e2.nome) ? -1 : 1;
            });
        }
        else if (por === 'sigla') {
            return lst.slice(0).sort((e1, e2) => {
                if (e1.sigla === e2.sigla) {
                    return 0;
                }
                return (e1.sigla < e2.sigla) ? -1 : 1;
            });
        }
    }
}
LstEstadoDirective.decorators = [
    { type: Directive, args: [{
                // tslint:disable-next-line:directive-selector
                selector: '[ngBrToolsListEstados]'
            },] },
];
/** @nocollapse */
LstEstadoDirective.ctorParameters = () => [
    { type: ElementRef, },
    { type: undefined, decorators: [{ type: Inject, args: [ESTADO_SERVICE,] },] },
];
LstEstadoDirective.propDecorators = {
    "orderBy": [{ type: Input, args: ['ngBrToolsOrderBy',] },],
    "onEstados": [{ type: Output, args: ['ngBrToolsListEstados',] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class Estado {
    /**
     * @param {?=} nome
     * @param {?=} sigla
     * @param {?=} bandeira
     */
    constructor(nome, sigla, bandeira) {
        this.nome = nome;
        this.sigla = sigla;
        this.bandeira = bandeira;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class CepPatternDirective extends MaskPatternDirective {
    /**
     * @return {?}
     */
    ngOnInit() {
        this.pattern = CEP_MASK;
        super.ngOnInit();
    }
}
CepPatternDirective.decorators = [
    { type: Directive, args: [{
                // tslint:disable-next-line:directive-selector
                selector: '[ngBrToolsCepPattern]'
            },] },
];
/** @nocollapse */
CepPatternDirective.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class CepPatternPipe {
    /**
     * @param {?} value
     * @return {?}
     */
    transform(value) {
        return StringFormatter.maskedFormatter(value, CEP_MASK);
    }
}
CepPatternPipe.decorators = [
    { type: Pipe, args: [{
                name: 'ngBrToolsCepPatternPipe'
            },] },
];
/** @nocollapse */
CepPatternPipe.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
let CEP_SERVICE = new InjectionToken('cep.service');
let CepServiceFactory = (cepService) => {
    cepService.init();
    return cepService;
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class CepSearchDirective {
    /**
     * @param {?} cepEl
     * @param {?} cepService
     */
    constructor(cepEl, cepService) {
        this.cepEl = cepEl;
        // tslint:disable-next-line:no-input-rename
        this.onEndereco = new EventEmitter();
        this.onError = new EventEmitter();
        this._cepService = cepService;
    }
    /**
     * @param {?} e
     * @return {?}
     */
    onInput(e) {
        const /** @type {?} */ cepInput = (/** @type {?} */ (e.target));
        let /** @type {?} */ pattern;
        if (cepInput.hasAttribute('ngBrToolsMaskPattern')) {
            pattern = cepInput.getAttribute('ngBrToolsMaskPattern');
        }
        else {
            pattern = CEP_MASK;
        }
        const /** @type {?} */ cepClear = StringFormatter.clearFormat(cepInput.value, pattern);
        if (cepClear.length === 8) {
            this.doSearch(cepClear);
        }
    }
    /**
     * @param {?} cep
     * @return {?}
     */
    doSearch(cep) {
        const /** @type {?} */ resp = this._cepService.buscaCep(cep);
        if (resp instanceof Observable) {
            (/** @type {?} */ (resp)).subscribe((e) => {
                this.onEndereco.emit(e);
            }, (err) => {
                this.onError.emit(err);
            });
        }
        else {
            (/** @type {?} */ (resp))
                .then((e) => this.onEndereco.emit(e))
                .catch((err) => {
                this.onError.emit(err);
            });
        }
    }
}
CepSearchDirective.decorators = [
    { type: Directive, args: [{
                // tslint:disable-next-line:directive-selector
                selector: '[ngBrToolsCepSearch]'
            },] },
];
/** @nocollapse */
CepSearchDirective.ctorParameters = () => [
    { type: ElementRef, },
    { type: undefined, decorators: [{ type: Inject, args: [CEP_SERVICE,] },] },
];
CepSearchDirective.propDecorators = {
    "onEndereco": [{ type: Output },],
    "onError": [{ type: Output },],
    "onInput": [{ type: HostListener, args: ['input', ['$event'],] },],
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class Endereco {
    /**
     * @param {?=} endereco
     * @param {?=} bairro
     * @param {?=} cep
     * @param {?=} cidade
     * @param {?=} complemento
     * @param {?=} complemento2
     * @param {?=} uf
     * @param {?=} id
     */
    constructor(endereco, bairro, cep, cidade, complemento, complemento2, uf, id) {
        this.endereco = endereco;
        this.bairro = bairro;
        this.cep = cep;
        this.cidade = cidade;
        this.complemento = complemento;
        this.complemento2 = complemento2;
        this.uf = uf;
        this.id = id;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class NgBrToolsModule {
}
NgBrToolsModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    MaskPatternDirective,
                    CepPatternDirective,
                    CepSearchDirective,
                    CpfPatternDirective,
                    CnpjPatternDirective,
                    MaskPatternPipe,
                    CepPatternPipe,
                    CpfPatternPipe,
                    CnpjPatternPipe,
                    CPFValidator,
                    CNPJValidator,
                    LstEstadoDirective,
                ],
                imports: [
                    CommonModule
                ],
                providers: [],
                exports: [
                    MaskPatternDirective,
                    CepPatternDirective,
                    CepSearchDirective,
                    CpfPatternDirective,
                    CnpjPatternDirective,
                    MaskPatternPipe,
                    CepPatternPipe,
                    CpfPatternPipe,
                    CnpjPatternPipe,
                    CPFValidator,
                    CNPJValidator,
                    LstEstadoDirective,
                ],
            },] },
];
/** @nocollapse */
NgBrToolsModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */

export { MaskPatternDirective, MaskPatternPipe, CpfPatternDirective, CpfPatternPipe, CPFValidator, CnpjPatternDirective, CnpjPatternPipe, CNPJValidator, LstEstadoDirective, ESTADO_SERVICE, EstadoServiceFactory, Estado, CepPatternDirective, CepPatternPipe, CepSearchDirective, CEP_SERVICE, CepServiceFactory, Endereco, NgBrToolsModule };
//# sourceMappingURL=ng-br-tools.js.map
