import { __values, __extends } from 'tslib';
import { Directive, Input, HostListener, ElementRef, Pipe, forwardRef, InjectionToken, Inject, EventEmitter, Output, NgModule } from '@angular/core';
import { NG_VALIDATORS } from '@angular/forms';
import { Observable } from 'rxjs/Observable';
import 'rxjs/add/operator/map';
import 'rxjs/add/observable/fromPromise';
import { CommonModule } from '@angular/common';

var CEP_MASK = '_____-___';
var CNPJ_MASK = '__.___.___/____-__';
var CPF_MASK = '___.___.___-__';
var StringFormatter = /** @class */ (function () {
    function StringFormatter() {
    }
    StringFormatter.maskedFormatter = function (input, mask) {
        var maskChr = StringFormatter.getMostCommonChar(mask);
        var maskArr = mask.split('');
        var inputArr = StringFormatter.clearFormat(input, mask).split('');
        var currMaskChar = '';
        var newValue = '';
        while (inputArr.length > 0 && maskArr.length > 0) {
            currMaskChar = maskArr.shift();
            if (currMaskChar === maskChr) {
                newValue += inputArr.shift();
            }
            else {
                newValue += currMaskChar;
            }
        }
        return newValue + maskArr.join('');
    };
    StringFormatter.clearFormat = function (str, mask) {
        var maskChr = StringFormatter.getMostCommonChar(mask);
        var maskStr = mask.replace(maskChr, '').split('');
        var retVal = str;
        try {
            for (var maskStr_1 = __values(maskStr), maskStr_1_1 = maskStr_1.next(); !maskStr_1_1.done; maskStr_1_1 = maskStr_1.next()) {
                var c = maskStr_1_1.value;
                retVal = retVal.replace(c, '');
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (maskStr_1_1 && !maskStr_1_1.done && (_a = maskStr_1.return)) _a.call(maskStr_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return retVal;
        var e_1, _a;
    };
    StringFormatter.getMostCommonChar = function (str) {
        var strCountArr = new Map();
        var maxKey = '';
        try {
            for (var str_1 = __values(str), str_1_1 = str_1.next(); !str_1_1.done; str_1_1 = str_1.next()) {
                var key = str_1_1.value;
                if (!strCountArr.has(key)) {
                    strCountArr.set(key, 1);
                }
                else {
                    strCountArr.set(key, strCountArr.get(key) + 1);
                }
                if (maxKey === '' || strCountArr.get(maxKey) < strCountArr.get(key)) {
                    maxKey = key;
                }
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (str_1_1 && !str_1_1.done && (_a = str_1.return)) _a.call(str_1);
            }
            finally { if (e_2) throw e_2.error; }
        }
        return maxKey;
        var e_2, _a;
    };
    return StringFormatter;
}());
var MaskPatternDirective = /** @class */ (function () {
    function MaskPatternDirective(el) {
        this.el = el;
        this.showPlaceholder = true;
        this.patternStr = [];
        this.patternArr = [];
    }
    MaskPatternDirective.prototype.ngOnInit = function () {
        if (this.showPlaceholder && !this.el.nativeElement.placeholder) {
            this.el.nativeElement.placeholder = this.pattern;
        }
    };
    MaskPatternDirective.prototype.onInput = function (event) {
        var input = ((event.target));
        input.value = StringFormatter.maskedFormatter(input.value, this.pattern);
    };
    return MaskPatternDirective;
}());
MaskPatternDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ngBrToolsMaskPattern]'
            },] },
];
MaskPatternDirective.ctorParameters = function () { return [
    { type: ElementRef, },
]; };
MaskPatternDirective.propDecorators = {
    "pattern": [{ type: Input, args: ['ngBrToolsMaskPattern',] },],
    "showPlaceholder": [{ type: Input, args: ['ngBrToolsShowPlacholder',] },],
    "onInput": [{ type: HostListener, args: ['input', ['$event'],] },],
};
var MaskPatternPipe = /** @class */ (function () {
    function MaskPatternPipe() {
    }
    MaskPatternPipe.prototype.transform = function (value, mask) {
        return StringFormatter.maskedFormatter(value, mask);
    };
    return MaskPatternPipe;
}());
MaskPatternPipe.decorators = [
    { type: Pipe, args: [{
                name: 'ngBrToolsMaskPatternPipe'
            },] },
];
MaskPatternPipe.ctorParameters = function () { return []; };
var CpfPatternDirective = /** @class */ (function (_super) {
    __extends(CpfPatternDirective, _super);
    function CpfPatternDirective() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    CpfPatternDirective.prototype.ngOnInit = function () {
        this.pattern = CPF_MASK;
        _super.prototype.ngOnInit.call(this);
    };
    return CpfPatternDirective;
}(MaskPatternDirective));
CpfPatternDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ngBrToolsCpfPattern]'
            },] },
];
CpfPatternDirective.ctorParameters = function () { return []; };
var CpfPatternPipe = /** @class */ (function () {
    function CpfPatternPipe() {
    }
    CpfPatternPipe.prototype.transform = function (value) {
        return StringFormatter.maskedFormatter(value, CPF_MASK);
    };
    return CpfPatternPipe;
}());
CpfPatternPipe.decorators = [
    { type: Pipe, args: [{
                name: 'ngBrToolsCpfPatternPipe'
            },] },
];
CpfPatternPipe.ctorParameters = function () { return []; };
var BrValidator = /** @class */ (function () {
    function BrValidator() {
    }
    BrValidator.validaCpf = function (cpf) {
        var cpfAux = '';
        var digito1 = '';
        var digito2 = '';
        var brValidatorUtils = new BrValidatotUtils(cpf);
        if (!brValidatorUtils.isValid()) {
            return false;
        }
        cpfAux = brValidatorUtils.removeDigitosVerificador();
        digito1 = brValidatorUtils.calculaDigitoVerificador(cpfAux);
        digito2 = brValidatorUtils.calculaDigitoVerificador(cpfAux + digito1);
        return cpf === cpfAux + digito1 + digito2;
    };
    BrValidator.validaCnpj = function (cnpj) {
        var cnpjAux = '';
        var digito1 = '';
        var digito2 = '';
        var brValidatorUtils = new BrValidatotUtils(cnpj);
        if (!brValidatorUtils.isValid()) {
            return false;
        }
        cnpjAux = brValidatorUtils.removeDigitosVerificador();
        digito1 = brValidatorUtils.calculaDigitoVerificador(cnpjAux);
        digito2 = brValidatorUtils.calculaDigitoVerificador(cnpjAux + digito1);
        return cnpj === cnpjAux + digito1 + digito2;
    };
    return BrValidator;
}());
var BrValidatotUtils = /** @class */ (function () {
    function BrValidatotUtils(str) {
        this.str = str;
    }
    BrValidatotUtils.prototype.isValid = function () {
        return this.checkCanBeValid(this.str, 'cpf') ||
            this.checkCanBeValid(this.str, 'cnpj');
    };
    BrValidatotUtils.prototype.calculaDigitoVerificador = function (str) {
        var tblPesos = [];
        var digito;
        var somatorio = 0;
        if (str.length === 9) {
            tblPesos = [10, 9, 8, 7, 6, 5, 4, 3, 2];
        }
        else if (str.length === 10) {
            tblPesos = [11, 10, 9, 8, 7, 6, 5, 4, 3, 2];
        }
        else if (str.length === 12) {
            tblPesos = [5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2];
        }
        else if (str.length === 13) {
            tblPesos = [6, 5, 4, 3, 2, 9, 8, 7, 6, 5, 4, 3, 2];
        }
        var i = 0;
        try {
            for (var tblPesos_1 = __values(tblPesos), tblPesos_1_1 = tblPesos_1.next(); !tblPesos_1_1.done; tblPesos_1_1 = tblPesos_1.next()) {
                var p = tblPesos_1_1.value;
                somatorio += p * Number(str.charAt(i));
                i++;
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (tblPesos_1_1 && !tblPesos_1_1.done && (_a = tblPesos_1.return)) _a.call(tblPesos_1);
            }
            finally { if (e_3) throw e_3.error; }
        }
        var resto = somatorio % 11;
        digito = 11 - resto;
        if (digito > 9) {
            digito = 0;
        }
        return digito.toString();
        var e_3, _a;
    };
    BrValidatotUtils.prototype.removeDigitosVerificador = function () {
        return this.str.substring(0, this.str.length - 2);
    };
    BrValidatotUtils.prototype.checkCanBeValid = function (value, opt) {
        if (value == null || value.length === 0) {
            return false;
        }
        switch (opt) {
            case 'cpf':
                return this.canBeValidCpf(value);
            case 'cnpj':
                return this.canBeValidCnpj(value);
        }
        return false;
    };
    BrValidatotUtils.prototype.canBeValidCpf = function (cpf) {
        if (cpf.length !== 11) {
            return false;
        }
        if ((cpf === '00000000000') ||
            (cpf === '11111111111') ||
            (cpf === '22222222222') ||
            (cpf === '33333333333') ||
            (cpf === '44444444444') ||
            (cpf === '55555555555') ||
            (cpf === '66666666666') ||
            (cpf === '77777777777') ||
            (cpf === '88888888888') ||
            (cpf === '99999999999')) {
            return false;
        }
        return true;
    };
    BrValidatotUtils.prototype.canBeValidCnpj = function (cnpj) {
        if (cnpj.length !== 14) {
            return false;
        }
        if ((cnpj === '00000000000000') ||
            (cnpj === '11111111111111') ||
            (cnpj === '22222222222222') ||
            (cnpj === '33333333333333') ||
            (cnpj === '44444444444444') ||
            (cnpj === '55555555555555') ||
            (cnpj === '66666666666666') ||
            (cnpj === '77777777777777') ||
            (cnpj === '88888888888888') ||
            (cnpj === '99999999999999')) {
            return false;
        }
        return true;
    };
    return BrValidatotUtils;
}());
var CPFValidator = /** @class */ (function () {
    function CPFValidator() {
    }
    CPFValidator.prototype.validate = function (c) {
        if (c.value && c.value.length > 0) {
            var regex = /\.|-/gi;
            var value = c.value.replace(regex, '');
            if (BrValidator.validaCpf(value)) {
                return null;
            }
        }
        return {
            CPFValidator: {
                valid: false
            }
        };
    };
    return CPFValidator;
}());
CPFValidator.decorators = [
    { type: Directive, args: [{
                selector: '[ngBrToolsCpfValidator][ngModel],[ngBrToolsCpfValidator][formControl],[ngBrToolsCpfValidator][formControlName]',
                providers: [
                    {
                        provide: NG_VALIDATORS,
                        useExisting: forwardRef(function () { return CPFValidator; }),
                        multi: true
                    }
                ]
            },] },
];
CPFValidator.ctorParameters = function () { return []; };
var CnpjPatternDirective = /** @class */ (function (_super) {
    __extends(CnpjPatternDirective, _super);
    function CnpjPatternDirective() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    CnpjPatternDirective.prototype.ngOnInit = function () {
        this.pattern = CNPJ_MASK;
        _super.prototype.ngOnInit.call(this);
    };
    return CnpjPatternDirective;
}(MaskPatternDirective));
CnpjPatternDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ngBrToolsCnpjPattern]'
            },] },
];
CnpjPatternDirective.ctorParameters = function () { return []; };
var CnpjPatternPipe = /** @class */ (function () {
    function CnpjPatternPipe() {
    }
    CnpjPatternPipe.prototype.transform = function (value) {
        return StringFormatter.maskedFormatter(value, CNPJ_MASK);
    };
    return CnpjPatternPipe;
}());
CnpjPatternPipe.decorators = [
    { type: Pipe, args: [{
                name: 'ngBrToolsCnpjPatternPipe'
            },] },
];
CnpjPatternPipe.ctorParameters = function () { return []; };
var CNPJValidator = /** @class */ (function () {
    function CNPJValidator() {
    }
    CNPJValidator.prototype.validate = function (c) {
        if (c.value && c.value.length > 0) {
            var regex = /\.|-|\//gi;
            var value = c.value.replace(regex, '');
            if (BrValidator.validaCnpj(value)) {
                return null;
            }
        }
        return {
            CNPJValidator: {
                valid: false
            }
        };
    };
    return CNPJValidator;
}());
CNPJValidator.decorators = [
    { type: Directive, args: [{
                selector: '[ngBrToolsCnpjValidator][ngModel],[ngBrToolsCnpjValidator][formControl],[ngBrToolsCnpjValidator][formControlName]',
                providers: [
                    {
                        provide: NG_VALIDATORS,
                        useExisting: forwardRef(function () { return CNPJValidator; }),
                        multi: true
                    }
                ]
            },] },
];
CNPJValidator.ctorParameters = function () { return []; };
var ESTADO_SERVICE = new InjectionToken('estado.service');
var EstadoServiceFactory = function (estadoService) {
    estadoService.init();
    return estadoService;
};
var LstEstadoDirective = /** @class */ (function () {
    function LstEstadoDirective(el, estadoService) {
        this.el = el;
        this.orderBy = 'nome';
        this.onEstados = new EventEmitter();
        this._estadoService = estadoService;
    }
    LstEstadoDirective.prototype.ngOnInit = function () {
        var _this = this;
        var resp = this._estadoService.buscaEstados();
        if (resp instanceof Promise) {
            resp = Observable.fromPromise(((resp)));
        }
        ((resp)).map(function (estados) {
            return _this._sortEstados(estados, _this.orderBy);
        }).subscribe(function (e) { return _this.onEstados.emit(e); });
    };
    LstEstadoDirective.prototype._sortEstados = function (lst, por) {
        if (por === void 0) { por = 'nome'; }
        if (lst.length <= 1) {
            return lst;
        }
        if (por === 'nome') {
            return lst.slice(0).sort(function (e1, e2) {
                if (e1.nome === e2.nome) {
                    return 0;
                }
                return (e1.nome < e2.nome) ? -1 : 1;
            });
        }
        else if (por === 'sigla') {
            return lst.slice(0).sort(function (e1, e2) {
                if (e1.sigla === e2.sigla) {
                    return 0;
                }
                return (e1.sigla < e2.sigla) ? -1 : 1;
            });
        }
    };
    return LstEstadoDirective;
}());
LstEstadoDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ngBrToolsListEstados]'
            },] },
];
LstEstadoDirective.ctorParameters = function () { return [
    { type: ElementRef, },
    { type: undefined, decorators: [{ type: Inject, args: [ESTADO_SERVICE,] },] },
]; };
LstEstadoDirective.propDecorators = {
    "orderBy": [{ type: Input, args: ['ngBrToolsOrderBy',] },],
    "onEstados": [{ type: Output, args: ['ngBrToolsListEstados',] },],
};
var Estado = /** @class */ (function () {
    function Estado(nome, sigla, bandeira) {
        this.nome = nome;
        this.sigla = sigla;
        this.bandeira = bandeira;
    }
    return Estado;
}());
var CepPatternDirective = /** @class */ (function (_super) {
    __extends(CepPatternDirective, _super);
    function CepPatternDirective() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    CepPatternDirective.prototype.ngOnInit = function () {
        this.pattern = CEP_MASK;
        _super.prototype.ngOnInit.call(this);
    };
    return CepPatternDirective;
}(MaskPatternDirective));
CepPatternDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ngBrToolsCepPattern]'
            },] },
];
CepPatternDirective.ctorParameters = function () { return []; };
var CepPatternPipe = /** @class */ (function () {
    function CepPatternPipe() {
    }
    CepPatternPipe.prototype.transform = function (value) {
        return StringFormatter.maskedFormatter(value, CEP_MASK);
    };
    return CepPatternPipe;
}());
CepPatternPipe.decorators = [
    { type: Pipe, args: [{
                name: 'ngBrToolsCepPatternPipe'
            },] },
];
CepPatternPipe.ctorParameters = function () { return []; };
var CEP_SERVICE = new InjectionToken('cep.service');
var CepServiceFactory = function (cepService) {
    cepService.init();
    return cepService;
};
var CepSearchDirective = /** @class */ (function () {
    function CepSearchDirective(cepEl, cepService) {
        this.cepEl = cepEl;
        this.onEndereco = new EventEmitter();
        this.onError = new EventEmitter();
        this._cepService = cepService;
    }
    CepSearchDirective.prototype.onInput = function (e) {
        var cepInput = ((e.target));
        var pattern;
        if (cepInput.hasAttribute('ngBrToolsMaskPattern')) {
            pattern = cepInput.getAttribute('ngBrToolsMaskPattern');
        }
        else {
            pattern = CEP_MASK;
        }
        var cepClear = StringFormatter.clearFormat(cepInput.value, pattern);
        if (cepClear.length === 8) {
            this.doSearch(cepClear);
        }
    };
    CepSearchDirective.prototype.doSearch = function (cep) {
        var _this = this;
        var resp = this._cepService.buscaCep(cep);
        if (resp instanceof Observable) {
            ((resp)).subscribe(function (e) {
                _this.onEndereco.emit(e);
            }, function (err) {
                _this.onError.emit(err);
            });
        }
        else {
            ((resp))
                .then(function (e) { return _this.onEndereco.emit(e); })
                .catch(function (err) {
                _this.onError.emit(err);
            });
        }
    };
    return CepSearchDirective;
}());
CepSearchDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ngBrToolsCepSearch]'
            },] },
];
CepSearchDirective.ctorParameters = function () { return [
    { type: ElementRef, },
    { type: undefined, decorators: [{ type: Inject, args: [CEP_SERVICE,] },] },
]; };
CepSearchDirective.propDecorators = {
    "onEndereco": [{ type: Output },],
    "onError": [{ type: Output },],
    "onInput": [{ type: HostListener, args: ['input', ['$event'],] },],
};
var Endereco = /** @class */ (function () {
    function Endereco(endereco, bairro, cep, cidade, complemento, complemento2, uf, id) {
        this.endereco = endereco;
        this.bairro = bairro;
        this.cep = cep;
        this.cidade = cidade;
        this.complemento = complemento;
        this.complemento2 = complemento2;
        this.uf = uf;
        this.id = id;
    }
    return Endereco;
}());
var NgBrToolsModule = /** @class */ (function () {
    function NgBrToolsModule() {
    }
    return NgBrToolsModule;
}());
NgBrToolsModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    MaskPatternDirective,
                    CepPatternDirective,
                    CepSearchDirective,
                    CpfPatternDirective,
                    CnpjPatternDirective,
                    MaskPatternPipe,
                    CepPatternPipe,
                    CpfPatternPipe,
                    CnpjPatternPipe,
                    CPFValidator,
                    CNPJValidator,
                    LstEstadoDirective,
                ],
                imports: [
                    CommonModule
                ],
                providers: [],
                exports: [
                    MaskPatternDirective,
                    CepPatternDirective,
                    CepSearchDirective,
                    CpfPatternDirective,
                    CnpjPatternDirective,
                    MaskPatternPipe,
                    CepPatternPipe,
                    CpfPatternPipe,
                    CnpjPatternPipe,
                    CPFValidator,
                    CNPJValidator,
                    LstEstadoDirective,
                ],
            },] },
];
NgBrToolsModule.ctorParameters = function () { return []; };

export { MaskPatternDirective, MaskPatternPipe, CpfPatternDirective, CpfPatternPipe, CPFValidator, CnpjPatternDirective, CnpjPatternPipe, CNPJValidator, LstEstadoDirective, ESTADO_SERVICE, EstadoServiceFactory, Estado, CepPatternDirective, CepPatternPipe, CepSearchDirective, CEP_SERVICE, CepServiceFactory, Endereco, NgBrToolsModule };
//# sourceMappingURL=ng-br-tools.js.map
